var dtContextMenu = {

  // ID's of the context menu items used by Duplicate Tab
  IDS:
  {
    MENU_ITEM_IDS: ['duptab-separator', 'duplicate-tab-menu', 'duptab-new-tab', 'duptab-new-window', 'duptab-merge-window'],
    TAB_MENU_ITEM_IDS: ['tab-duptab-separator', 'tab-duplicate-tab-menu', 'tab-duptab-new-tab', 'tab-duptab-new-window', 'tab-duptab-merge-window']
  },

  // add contextmenuitems to tab
  addTabContextMenu: function()
  {
    var tabbarContextMenu = getBrowser().mStrip.childNodes[1];

    for (var i=4; i>=0; i--) {
      var newItem = document.getElementById(dtContextMenu.IDS.MENU_ITEM_IDS[i]).cloneNode(true);
      newItem.setAttribute('id', dtContextMenu.IDS.TAB_MENU_ITEM_IDS[i]);
      tabbarContextMenu.insertBefore(newItem, tabbarContextMenu.childNodes[5]);
    }
    dtContextMenu.setPlaceTabContextMenuItems();

    document.getElementById('contentAreaContextMenu').addEventListener("popupshowing", dtContextMenu.contentContextShow, false);
    document.getElementById('contentAreaContextMenu').addEventListener("popuphiding", dtContextMenu.contextHide, false);
    tabbarContextMenu.addEventListener("popupshowing", dtContextMenu.tabContextShow, false);
    tabbarContextMenu.addEventListener("popuphiding", dtContextMenu.contextHide, false);
  },

  deinit: function()
  {
    document.getElementById('contentAreaContextMenu').removeEventListener("popupshowing", dtContextMenu.contentContextShow, false);
    document.getElementById('contentAreaContextMenu').removeEventListener("popuphiding", dtContextMenu.contextHide, false);

    var tabbarContextMenu = getBrowser().mStrip.childNodes[1];
    tabbarContextMenu.removeEventListener("popupshowing", dtContextMenu.tabContextShow, false);
    tabbarContextMenu.removeEventListener("popuphiding", dtContextMenu.contextHide, false);
  },

  positionListener:
  {
    observe: function(subject, topic, prefName)
    {
      if (topic != "nsPref:changed")
        return;
      dtContextMenu.setPlaceTabContextMenuItems();
    }
  },

  setPlaceTabContextMenuItems: function()
  {
    var menuPosition = duplicateTab.prefService.getIntPref("duplicatetab.contextMenuItems.position");
    var tabbarContextMenu = getBrowser().mStrip.childNodes[1];
    var menuItems = new Array();
    var dtMenuItems = tabbarContextMenu.getElementsByAttribute('duplicatetab', 'true');

    var i, j = 4;
    for (i=4; i>=0; i--) {
      menuItem = dtMenuItems[i];
      if ((menuItem.nodeName == 'menuitem') || (menuItem.nodeName == 'menu')) {
        menuItems[j] = tabbarContextMenu.removeChild(menuItem);
        j--;
      }
      else if (menuItem.nodeName == 'menuseparator')
        menuItems[0] = tabbarContextMenu.removeChild(menuItem);
    }

    if (menuPosition > tabbarContextMenu.childNodes.length-1) {
      for (i=4; i>=0; i--) {
        tabbarContextMenu.appendChild(menuItems[i]);
      }
      return;
    }

    if (((menuPosition == 0) || ((menuPosition > 0) && (tabbarContextMenu.childNodes[menuPosition-1].localName != 'menuitem'))) && (tabbarContextMenu.childNodes[menuPosition].localName == 'menuitem'))
      tabbarContextMenu.insertBefore(menuItems[0], tabbarContextMenu.childNodes[menuPosition]);

    for (i=4; i>=1; i--) {
      tabbarContextMenu.insertBefore(menuItems[i], tabbarContextMenu.childNodes[menuPosition]);
    }
    if ((menuPosition > 0) && (tabbarContextMenu.childNodes[menuPosition-1].localName == 'menuitem'))
      tabbarContextMenu.insertBefore(menuItems[i], tabbarContextMenu.childNodes[menuPosition]);
  },

  // only show the menuitems the user wishes to see
  showMenuItems: function(where, hideMenu)
  {
    var i, contextObj, menuitemids, subMenuPref;

    // disable window merging if only one window is open
    function blockMerging()
    {
      var i = 0;
      var winMediator = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
      var winEnum = winMediator.getEnumerator('navigator:browser');
      while(winEnum.hasMoreElements()) {
        var win = winEnum.getNext();
        i++;
      }
      if (i < 2)
        document.getElementById('cmd_mergeWindows').setAttribute('disabled', 'true');
      else
        document.getElementById('cmd_mergeWindows').removeAttribute('disabled');
    }

    function hideAllItems()
    {
      for (i=0; i<5; i++) {
        contextObj.getElementsByAttribute('id', menuitemids[i])[0].hidden = true;
      }
    }

    function hideSomeItems()
    {
      contextObj.getElementsByAttribute('id', menuitemids[0])[0].hidden = hideMenu;
      if (subMenuPref) {
        contextObj.getElementsByAttribute('id', menuitemids[1])[0].hidden = hideMenu;
        for (i=2; i<5; i++) {
          contextObj.getElementsByAttribute('id', menuitemids[i])[0].hidden = true;
        }
      }
      else {
        var whichMenuItems = duplicateTab.prefService.getCharPref("duplicatetab.showContextMenu.items");
        var dtItems;
        if (contextObj == document)
          dtItems = [Boolean(parseInt(whichMenuItems.charAt(0))>1), Boolean(parseInt(whichMenuItems.charAt(1))>1), Boolean(parseInt(whichMenuItems.charAt(2))>1)];
        else
          dtItems = [Boolean(parseInt(whichMenuItems.charAt(0))%2), Boolean(parseInt(whichMenuItems.charAt(1))%2), Boolean(parseInt(whichMenuItems.charAt(2))%2)];
        contextObj.getElementsByAttribute('id', menuitemids[1])[0].hidden = true;
        for (i=0; i<3; i++) {
          if (dtItems[i])
              contextObj.getElementsByAttribute('id', menuitemids[i+2])[0].hidden = hideMenu;
          else
            contextObj.getElementsByAttribute('id', menuitemids[i+2])[0].hidden = true;
        }
      }
    }

    blockMerging();

    switch (where) {

      // when the context menu of a tab is requested
      case 'tab':
        contextObj = getBrowser().mStrip.childNodes[1];
        menuitemids = dtContextMenu.IDS.TAB_MENU_ITEM_IDS;
        subMenuPref = duplicateTab.prefService.getBoolPref("duplicatetab.showContextMenu.inSubMenu");
        var inTabContext = duplicateTab.prefService.getBoolPref("duplicatetab.showContextMenu.onTab");
        if (!inTabContext) {
          hideAllItems();
          break;
        }
        else if (inTabContext) {
          hideSomeItems();
          break;
        }

      // when the context menu of a page is requested
      case 'content':
        contextObj = document;
        menuitemids = dtContextMenu.IDS.MENU_ITEM_IDS;
        subMenuPref = duplicateTab.prefService.getBoolPref("duplicatetab.showContextMenu.inContentSubMenu");
        var inPageContext = duplicateTab.prefService.getBoolPref("duplicatetab.showContextMenu.onContentArea");
        if (!inPageContext){
          hideAllItems();
          break;
        }
        else if (inPageContext) {
          hideSomeItems();
          break;
        }
    }
  },

  // set which context menu items to show and where
  contentContextShow: function()
  {
    var hiddenMenu = (gContextMenu.isTextSelected || gContextMenu.onTextInput);
    document.getElementById('duptab-open-link').hidden = !(gContextMenu.onSaveableLink || (gContextMenu.inDirList && gContextMenu.onLink));

    dtContextMenu.showMenuItems('content', hiddenMenu);

    return;
  },

  tabContextShow: function()
  {
    var popupNodeName = document.popupNode.localName;
    var duptabitems = getBrowser().mStrip.childNodes[1].getElementsByAttribute("duplicatetab", "true");

    // show menuitems alone in tabs context menu, not in the context menu of the tabbar
    if (popupNodeName != 'tab') {
      for (var i=0;i<duptabitems.length;i++) {
        duptabitems.item(i).hidden = true;
      }
    }

    // Hide the menuitems the user doesn't want to show
    else if (popupNodeName == 'tab')
      dtContextMenu.showMenuItems('tab', false);
  },

  contextHide: function()
  {
    document.getElementById('cmd_mergeWindows').removeAttribute('disabled');
  }
}